import { Component, Prop, h, State, Host, Listen } from '@stencil/core';
import axios from 'axios';
import { Block, MintStatus } from '../../block';
import { getBlockExplorer, handleError } from '../../utils/utils';
import { handleNewAccount } from '../../utils/wallet-helper';
export class StructCore {
  constructor() {
    /**
     * Flag to enable testnet (rinkeby)
     */
    this.testnet = false;
    /**
     *  Wallet connect button text
     */
    this.walletBtnText = 'Connect Wallet';
    /**
     * Mint button text
     */
    this.mintBtnText = 'Mint';
    /**
     * Sold out text
     */
    this.soldOutText = 'SOLD OUT!';
    /**
     * shows the remaining supply left
     */
    this.supplyLeft = false;
    /**
     * show the select button to select mint quantity
     */
    this.quantityInput = false;
    /**
     * show the status for the contract
     */
    this.saleStatus = false;
    /**
     * show the status for the contract
     */
    this.contractLink = false;
    /**
     * for devs only
     */
    this.server = 'https://struct--api.herokuapp.com/getWalletSignature';
    this.isMinting = false;
    this.isConnecting = false;
    this.selectedAmount = 1;
    this.totalSupply = 0;
    this.message = {};
    this.mint = async () => {
      try {
        this.message = {};
        this.isMinting = true;
        console.log('is minting');
        const tx = await this.block.mint(this.selectedAmount);
        this.message = { transaction: tx.hash };
        await tx.wait();
        this.message = { success: 'Successfully minted' };
        this.isMinting = false;
      }
      catch (error) {
        this.isMinting = false;
        this.message = { error: handleError(error) };
      }
    };
    this.getWalletSignature = async () => {
      var _a;
      try {
        const response = await axios.post(this.server, {
          contractAddress: this.contractAddress,
          wallet: this.block.walletInfo.signerAddress,
        }, {
          headers: {
            'Content-Type': 'application/json',
          },
        });
        const sig = response.data;
        if (sig.signature) {
          this.block.walletSignature = sig.signature;
          this.message = { info: 'Address whitelisted' };
        }
        else {
          this.message = { error: 'Address not whitelisted' };
        }
      }
      catch (e) {
        if (((_a = e === null || e === void 0 ? void 0 : e.response) === null || _a === void 0 ? void 0 : _a.status) === 400) {
          this.message = { error: 'Address not whitelisted' };
        }
      }
    };
    this.initiateWallet = async () => {
      try {
        this.block = undefined;
        await window.ethereum
          .request({ method: 'eth_requestAccounts' })
          .then(async (accounts) => {
          this.isConnecting = true;
          const info = {};
          await handleNewAccount(accounts, info);
          this.block = await Block.create(this.contractAddress, this.testnet, this.type, info);
          this.totalSupply = this.block.totalSupply || 0;
          if (this.block.status !== MintStatus.CLOSED) {
            // listen for mint events to update supply in real-time
            const filter = this.block.contract.filters.Transfer('0x0000000000000000000000000000000000000000');
            this.block.contract.on(filter, () => {
              this.totalSupply += 1;
            });
          }
          if (this.block.status === MintStatus.PRIVATE || this.block.status === MintStatus.BOTH) {
            await this.getWalletSignature();
          }
          this.initialized = true;
          this.isConnecting = false;
        })
          .catch(err => {
          console.log(err);
        });
      }
      catch (error) {
        console.log(error);
      }
    };
  }
  connectedCallback() {
    this.initiateWallet();
  }
  selectAmountChanged(event) {
    this.selectedAmount = event.detail;
  }
  render() {
    var _a, _b;
    return (h(Host, { class: "wrapper" },
      h("div", { part: "container", class: "container" },
        h("slot", null),
        this.saleStatus && h("mint-status", { "mint-status": this.block.status || MintStatus.CLOSED, exportparts: "status-bar, status-text" }),
        this.supplyLeft && h("supply-display", { "total-supply": this.totalSupply, "max-supply": (_a = this === null || this === void 0 ? void 0 : this.block) === null || _a === void 0 ? void 0 : _a.maxSupply, exportparts: "supply-bar, supply-left" }),
        h("div", { id: "message-box", part: "message-box" },
          this.block && ((_b = this.message) === null || _b === void 0 ? void 0 : _b.error) && (h("p", { class: "error-message", part: "error-message" }, this.message.error)),
          this.block && this.message.info && h("p", null, this.message.info),
          this.block && this.message.transaction && h("p", null, this.message.transaction),
          this.block && this.message.success && (h("p", { class: "success-message", part: "success-message" }, this.message.success))),
        this.initialized ? (h("div", { class: "form-control" }, this.isMinting && this.message.transaction !== undefined ? (h("button", { part: "etherscan-button", class: "etherscan-link", onClick: () => {
            window.open(`${this.testnet ? 'https://rinkeby.etherscan.io/tx/' : 'https://etherscan.io/tx/'}${this.message.transaction}`, '_blank');
          } })) : (h("div", { class: "input-group" },
          this.quantityInput && h("quantity-selector", { "selected-amount": this.selectedAmount, "allocation-left": this.block.personalAllocationLeft }),
          h("mint-button", { exportparts: "mint-button", disabled: this.block.status === MintStatus.CLOSED ||
              (this.block.status === MintStatus.PRIVATE && !this.block.walletSignature) ||
              this.block.personalAllocationLeft === 0, onClick: () => this.mint() }, !this.isMinting ? `${this.mintBtnText}` : `Minting ${this.selectedAmount}...`))))) : (h("wallet-button", { exportparts: "wallet-button", onClick: this.initiateWallet }, this.isConnecting ? 'Connecting...' : this.walletBtnText)),
        this.contractLink && (h("div", { class: "contract-link", part: "contract-link" },
          h("a", { target: "_blank", rel: "noreferrer", href: `${getBlockExplorer(this.testnet)}/address/${this.contractAddress}` }, "View Contract"))))));
  }
  static get is() { return "struct-core"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["struct-core.sass"]
  }; }
  static get styleUrls() { return {
    "$": ["struct-core.css"]
  }; }
  static get properties() { return {
    "contractAddress": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": true,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Deployed testnet or contract address"
      },
      "attribute": "contract-address",
      "reflect": false
    },
    "testnet": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Flag to enable testnet (rinkeby)"
      },
      "attribute": "testnet",
      "reflect": false,
      "defaultValue": "false"
    },
    "type": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "ContractType",
        "resolved": "\"ERC1155\" | \"ERC721\"",
        "references": {
          "ContractType": {
            "location": "import",
            "path": "../../block"
          }
        }
      },
      "required": true,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Struct contract type"
      },
      "attribute": "type",
      "reflect": false
    },
    "walletBtnText": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Wallet connect button text"
      },
      "attribute": "wallet-btn-text",
      "reflect": false,
      "defaultValue": "'Connect Wallet'"
    },
    "mintBtnText": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Mint button text"
      },
      "attribute": "mint-btn-text",
      "reflect": false,
      "defaultValue": "'Mint'"
    },
    "soldOutText": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Sold out text"
      },
      "attribute": "sold-out-text",
      "reflect": false,
      "defaultValue": "'SOLD OUT!'"
    },
    "supplyLeft": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "shows the remaining supply left"
      },
      "attribute": "supply-left",
      "reflect": false,
      "defaultValue": "false"
    },
    "quantityInput": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "show the select button to select mint quantity"
      },
      "attribute": "quantity-input",
      "reflect": false,
      "defaultValue": "false"
    },
    "saleStatus": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "show the status for the contract"
      },
      "attribute": "sale-status",
      "reflect": false,
      "defaultValue": "false"
    },
    "contractLink": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "show the status for the contract"
      },
      "attribute": "contract-link",
      "reflect": false,
      "defaultValue": "false"
    },
    "server": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "for devs only"
      },
      "attribute": "server",
      "reflect": false,
      "defaultValue": "'https://struct--api.herokuapp.com/getWalletSignature'"
    }
  }; }
  static get states() { return {
    "initialized": {},
    "isMinting": {},
    "isConnecting": {},
    "selectedAmount": {},
    "totalSupply": {},
    "message": {}
  }; }
  static get listeners() { return [{
      "name": "amountChanged",
      "method": "selectAmountChanged",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
